import os
import logging
from dataclasses import dataclass

from gpuc import PACKAGEDIR
from gpuc.lib.sops import Sops

logger = logging.getLogger(__name__)


@dataclass
class TmpConfig:
    """Configuration for the application."""

    gpuc_student_file: str


@dataclass
class GpucConfig:
    host: str
    port: int
    username: str
    password: str
    path: str
    ssh_ppk: str


@dataclass
class LdapmConfig:
    host: str
    port: int
    base: str
    username: str
    password: str


@dataclass
class MailConfig:
    host: str
    port: int
    login: str
    password: str
    sender: str
    receivers: list[str]

    def __post_init__(self):
        """Ensure receivers is a list."""
        if isinstance(self.receivers, str):
            self.receivers = self.receivers.split(",")
        elif not isinstance(self.receivers, list):
            raise TypeError("Receivers must be a list or a string.")
        if not self.receivers:
            raise ValueError("Receivers list cannot be empty.")


_gpuc_settings: GpucConfig | None = None
_mail_settings: MailConfig | None = None
_ldapm_settings: LdapmConfig | None = None
_tmp_settings: TmpConfig | None = None


def _get_settings(
    pathfile: str,
) -> tuple[GpucConfig, MailConfig, LdapmConfig, TmpConfig]:
    """Get the settings from the config file."""
    global _gpuc_settings, _mail_settings, _ldapm_settings, _tmp_settings

    logger.info(
        "Environnement utilisé : %s", os.getenv("GPUC_ENVIRONMENT", "dev").lower()
    )

    if (
        _gpuc_settings is None
        or _mail_settings is None
        or _ldapm_settings is None
        or _tmp_settings is None
    ):
        config = Sops.decrypt(pathfile)
        _gpuc_settings = GpucConfig(**config["server"]["gpuc"])
        _mail_settings = MailConfig(**config["mail"]["smtp"])
        _ldapm_settings = LdapmConfig(**config["server"]["ldapm"])
        _tmp_settings = TmpConfig(**config["app"]["tmp"])

    return _gpuc_settings, _mail_settings, _ldapm_settings, _tmp_settings


GPUC_SETTINGS, MAIL_SETTINGS, LDAPM_SETTINGS, TMP_SETTINGS = _get_settings(
    f"{PACKAGEDIR}/settings.{os.getenv('GPUC_ENVIRONMENT', 'dev').lower()}.enc.toml"
)
