from dataclasses import dataclass
from enum import Enum, unique
import tempfile
import csv
from pathlib import Path
import logging

from gpuc.lib.ldap.ldapm import LdapmClient
from gpuc.lib.msftp import SFTPClient
from gpuc import FOLDER_EMPLOYEE
from gpuc.config import GPUC_SETTINGS, TMP_SETTINGS


logger = logging.getLogger(__name__)


@unique
class Population(Enum):
    """Enumération des modes de traitement pour la génération des fichiers d'autorisation"""

    PERSONNEL_ONLY = "personnel_only"
    STUDENT_ONLY = "student_only"
    ALL = "all"

    def __str__(self):
        """Retourne le nom de l'énumération."""
        return self.value


@dataclass
class GenerateStatus:
    """Classe représentant le statut d'exécution de la fonction generate."""

    code: int

    @property
    def description(self) -> str:
        """Retourne une description fixe en fonction du code."""
        descriptions = {
            1: "Succès : fichiers générés et envoyés avec succès par SFTP.",
            2: "Succès : fichiers générés avec succès et sauvardés localement.",
            11: "Erreur : échec lors de l'envoi via SFTP.",
            12: "Erreur : échec lors de la génération des fichiers.",
        }
        return descriptions.get(self.code, "Erreur inconnue.")


@dataclass
class TPeople:
    """Classe pour la construction des fichiers d'authorisation pour GPUC"""

    @staticmethod
    def generate(
        *, population: Population = Population.PERSONNEL_ONLY, sftp: bool = False
    ) -> GenerateStatus:
        """
        Génération des fichiers d'autorisation pour GPUC et envoi via SFTP si demandé.

        Args:
            population (Population): Mode de traitement pour la génération des fichiers d'autorisation.
            sftp (bool): Si True, le fichier généré sera envoyé via SFTP.

        Returns:
            GenerateStatus: Statut de l'exécution.
        """
        try:
            # Récupération des données en fonction du population
            data = []
            if population in {Population.PERSONNEL_ONLY, Population.ALL}:
                employees = LdapmClient().get_all_employe()
                data.extend(
                    [
                        [
                            emp.supannAliaslogin,
                            emp.runUnivNom[0],  # Nom usuel
                            emp.runUnivPrenom[0],  # Prénom usuel
                            emp.mail,
                            emp.supannEntiteAffectationPrincipale,
                            "Personnel",
                        ]
                        for emp in employees
                    ]
                )
            if population in {Population.STUDENT_ONLY, Population.ALL}:
                logger.info(
                    "Récupération des étudiants depuis le fichier %s",
                    TMP_SETTINGS.gpuc_student_file,
                )
                if Path(TMP_SETTINGS.gpuc_student_file).exists():
                    logger.debug(
                        "Le fichier %s existe, lecture des données",
                        TMP_SETTINGS.gpuc_student_file,
                    )
                    with Path(TMP_SETTINGS.gpuc_student_file).open(
                        mode="r", encoding="utf-8"
                    ) as studfile:
                        reader = csv.reader(studfile, delimiter=";")
                        logger.info(
                            "Fusion des données des étudiants avec les données des employés dans le fichier temporaire"
                        )
                        data.extend(list(reader)[1:])  # Ignorer l'en-tête
                else:
                    logger.warning(
                        "Le fichier %s n'existe pas, aucune donnée d'étudiant n'a été ajoutée",
                        TMP_SETTINGS.gpuc_student_file,
                    )

            # Création du fichier temporaire
            temp_file = tempfile.NamedTemporaryFile(
                mode="w",
                newline="",
                encoding="utf-8",
                delete=False,
                prefix="gpuc-people-",
                dir=FOLDER_EMPLOYEE,
            )
            with Path(temp_file.name).open(
                mode="w", newline="", encoding="utf-8"
            ) as csvfile:
                writer = csv.writer(csvfile, delimiter=";")
                # Écriture de l'en-tête
                writer.writerow(
                    ["uid", "nom", "prenom", "mail", "code_parent", "statut"]
                )
                # Écriture des données
                writer.writerows(data)

            # Envoi via SFTP si demandé
            if sftp:
                sftp_client = SFTPClient(
                    hostname=GPUC_SETTINGS.host,
                    port=GPUC_SETTINGS.port,
                    username=GPUC_SETTINGS.username,
                    password=GPUC_SETTINGS.password,
                    private_key_path=GPUC_SETTINGS.ssh_ppk,
                )
                sftp_client.connect()
                remote_path = f"{GPUC_SETTINGS.path}/people-TEST.csv"
                sftp_client.upload_file(remote_path, temp_file.name)
                sftp_client.disconnect()
                logger.debug("Suppression du fichier temporaire")

                try:
                    Path(temp_file.name).unlink()
                except Exception as e:
                    logger.warning(
                        "Impossible de supprimer le fichier temporaire %s : %s",
                        temp_file.name,
                        e,
                    )

            rs = GenerateStatus(code=1 if sftp else 2)
            logger.info("Code retour pour la fonction **generate** : %s", rs.code)
            return rs

        except Exception as e:
            rs = GenerateStatus(code=11 if sftp else 12)
            logger.debug("Code retour pour la fonction **generate** : %s", rs.code)
            logger.exception(
                "Erreur lors de la génération ou de l'envoi des fichiers : %s", e
            )
            return rs
