from dataclasses import dataclass
from gpuc.config import LDAPM_SETTINGS
from gpuc.model.ldap.people import People
from gpuc.model.ldap.uo import UO
from typing import Generator
from .client import Client
from ldap3 import SUBTREE


import logging

logger = logging.getLogger(__name__)


@dataclass
class LdapmClient(Client):
    host: str = LDAPM_SETTINGS.host
    port: int = LDAPM_SETTINGS.port
    user: str = LDAPM_SETTINGS.username
    password: str = LDAPM_SETTINGS.password

    def __post_init__(self):
        """Initialise le serveur et la connexion LDAP."""
        super().__post_init__()

    def get_all_employe(self) -> Generator[People, None, None] | None:
        """Récupère tous les employés de l'annuaire LDAP.

        Returns:
            Generator: Générateur d'instances de la classe People représentant les employés.

        Raises:
            Exception: En cas d'erreur lors de la récupération des employés ou si aucun employé n'est trouvé.
        """
        try:
            base_dn = "ou=People,dc=univ-reunion,dc=fr"
            search_filter = (
                "(&(!(runUnivCategorie=tmp))(mail=*@univ-reunion.fr)(runUnivSihamId=*)"
                "(eduPersonAffiliation=employee)(supannAliasLogin=*)(supannEntiteAffectationPrincipale=HU*))"
            )
            attributes = [
                "cn",
                "sn",
                "runUnivNom",
                "runUnivPrenom",
                "supannAliaslogin",
                "mail",
                "runUnivSihamId",
                "runUnivAuthorization",
                "supannEntiteAffectationPrincipale",
            ]

            logger.info("Récupération des employés de l'annuaire LDAP")
            if (
                not self.connection.search(
                    search_base=base_dn,
                    search_filter=search_filter,
                    search_scope=SUBTREE,
                    attributes=attributes,
                )
                or not self.connection.entries
            ):
                logger.error("Aucun résultat trouvé dans l'annuaire LDAP")
                raise

            return (
                People(
                    cn=entry.cn.value,
                    sn=entry.sn.value,
                    runUnivNom=entry.runUnivNom.value,
                    runUnivPrenom=entry.runUnivPrenom.value,
                    supannAliaslogin=entry.supannAliasLogin.value,
                    mail=entry.mail.value,
                    runUnivSihamId=entry.runUnivSihamId.value,
                    runUnivAuthorization=entry.runUnivAuthorization.value,
                    supannEntiteAffectationPrincipale=entry.supannEntiteAffectationPrincipale.value,
                )
                for entry in self.connection.entries
            )
        except Exception as e:
            # Gérer les erreurs LDAP
            msgerr = f"Erreur lors de la récupération des employés : {e}"
            logger.debug(msgerr)
            raise
        finally:
            # Fermer la connexion LDAP
            self.close()

    def get_siham_uo(self) -> Generator[UO, None, None] | None:
        """Récupère toutes les structures hierarchiques de SIHAM

        Returns:
            Generator: Générateur d'instances de la classe UO représentant les structures.

        Raises:
            Exception: En cas d'erreur lors de la récupération des uo ou si aucun uo n'est trouvé.
        """

        try:
            base_dn = "ou=Structures,dc=univ-reunion,dc=fr"
            search_filter = "(supannRefId={SIHAM:STRUCTURE}*)"
            attributes = [
                "cn",
                "ou",
                "supannCodeEntite",
                "description",
                "supannCodeEntiteParent",
                "runUnivStrInfos",
                "supannRefId",
                "runUnivLastModified",
            ]
            logger.info("Récupération des employés de l'annuaire LDAP")
            if (
                not self.connection.search(
                    search_base=base_dn,
                    search_filter=search_filter,
                    search_scope=SUBTREE,
                    attributes=attributes,
                )
                or not self.connection.entries
            ):
                logger.error("Aucun résultat trouvé dans l'annuaire LDAP")
                raise

            return (
                UO(
                    cn=entry.cn.value,
                    ou=entry.ou.value,
                    supannCodeEntite=entry.supannCodeEntite.value,
                    description=entry.description.value,
                    runUnivLastModified=entry.runUnivLastModified.value
                    if entry.runUnivLastModified.value
                    else None,
                    runUnivStrInfos=entry.runUnivStrInfos.value,
                    supannCodeEntiteParent=entry.supannCodeEntiteParent.value,
                    supannRefId=entry.supannRefId.value,
                )
                for entry in self.connection.entries
            )
        except Exception as e:
            # Gérer les erreurs LDAP
            msgerr = f"Erreur lors de la récupération des structures : {e}"
            logger.debug(msgerr)
            raise
        finally:
            # Fermer la connexion LDAP
            self.close()
