import logging
import logging.handlers
import smtplib
import ssl


class BufferingSMTPHandler(logging.handlers.BufferingHandler):
    """
    Un gestionnaire de logs qui envoie des messages par e-mail en mode bufferisé.

    Attributes:
        mailhost (str): Adresse du serveur SMTP.
        mailport (int): Port du serveur SMTP.
        mailauth (str): Identifiant smtp.
        mailpwd (str): Mot de passe smtp.
        fromaddr (str): Adresse e-mail de l'expéditeur.
        toaddrs (list[str]): Liste des adresses e-mail des destinataires.
        subject (str): Sujet de l'e-mail.
        capacity (int): Taille maximale du buffer avant l'envoi des logs.
        tls (bool): Indique si TLS doit être activé pour la connexion SMTP.
    """

    def __init__(
        self,
        mailhost: str,
        mailport: int,
        mailauth: str,
        mailpwd: str,
        fromaddr: str,
        toaddrs: list[str],
        subject: str,
        capacity: int,
        tls: bool = False,
        custom_msg: str = "",
    ):
        logging.handlers.BufferingHandler.__init__(self, capacity)
        self.mailhost = mailhost
        self.mailport = mailport
        self.mailauth = mailauth
        self.mailpwd = mailpwd
        self.fromaddr = fromaddr
        self.toaddrs = toaddrs
        self.subject = subject
        self.tls = tls
        self.custom_msg = custom_msg
        self.setFormatter(
            logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
        )

    def add_custom_msg(self, custom_msg: str):
        """Ajoute un message personnalisé au mail.

        Args:
            custom_msg (str): Le message personnalisé à ajouter.
        """
        self.custom_msg += custom_msg

    def flush(self):
        """Envoie les logs accumulés dans le buffer par e-mail."""
        if len(self.buffer) > 0:
            try:
                context = ssl.create_default_context()
                with smtplib.SMTP(self.mailhost, self.mailport) as smtp:
                    if self.tls:
                        smtp.starttls(context=context)
                    if self.mailauth and self.mailpwd:
                        smtp.login(self.mailauth, self.mailpwd)
                    # Ajout des en-têtes pour spécifier l'encodage UTF-8
                    msg = (
                        f"From: {self.fromaddr}\r\n"
                        f"To: {','.join(self.toaddrs)}\r\n"
                        f"Subject: {self.subject}\r\n"
                        f"Content-Type: text/plain; charset=utf-8\r\n"
                        f"Content-Transfer-Encoding: 8bit\r\n\r\n"
                    )
                    if self.custom_msg:
                        msg += self.custom_msg + "\r\n"
                    msg += "---------------------------- Logs ---------------------------- :\r\n"
                    msg += "\r\n".join(self.format(record) for record in self.buffer)
                    smtp.sendmail(
                        self.fromaddr, self.toaddrs, msg.encode("utf-8")
                    )  # Encodage en UTF-8
            except Exception:
                self.handleError(None)  # Gère les erreurs sans record spécifique
            finally:
                self.buffer = []
