import subprocess
import shutil
import json
import tomllib
from pathlib import Path


class Sops:
    @staticmethod
    def is_sops_installed() -> bool:
        """Vérifie si la librairie sops est installée sur le système."""
        return shutil.which("sops") is not None

    @staticmethod
    def decrypt(encrypted_file: str) -> dict:
        """
        Déchiffre un fichier avec sops et retourne les données sous forme de dictionnaire.

        Args:
            encrypted_file (str): Chemin du fichier chiffré.

        Returns:
            dict: Contenu déchiffré du fichier.

        Raises:
            FileNotFoundError: Si le fichier chiffré n'existe pas.
            RuntimeError: Si une erreur se produit lors du déchiffrement.
        """
        if not Sops.is_sops_installed():
            raise RuntimeError("La librairie sops n'est pas installée sur le système.")

        try:
            result = subprocess.run(
                ["sops", "decrypt", encrypted_file],
                check=True,
                text=True,
                capture_output=True,
            )
            match Path(encrypted_file).suffix:
                case ".toml":
                    return tomllib.loads(result.stdout)
                case ".yaml" | ".yml":
                    # Il faut installer PyYAML pour utiliser cette fonctionnalité
                    raise "Le format YAML n'est pas supporté pour le moment."
                case _:
                    return json.loads(result.stdout)
            # return result.stdout
        except subprocess.CalledProcessError as e:
            raise RuntimeError(f"Erreur lors du déchiffrement du fichier : {e}")
        except json.JSONDecodeError as e:
            raise RuntimeError(f"Erreur lors de la conversion en dictionnaire : {e}")
